/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.jms.server;

import org.jboss.jms.server.endpoint.ConnectionEndpoint;

import java.util.List;


/**
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 * @author <a href="mailto:tim.fox@jboss.com">Tim Fox</a>
 * @version <tt>$Revision: 2591 $</tt>
 *
 * $Id: ConnectionManager.java 2591 2007-04-19 15:25:51Z timfox $
 */
public interface ConnectionManager
{
   void registerConnection(String jmsClientVMId, String remotingClientSessionID, ConnectionEndpoint endpoint);

   /**
    * @return null if there is no such connection.
    */
   ConnectionEndpoint unregisterConnection(String jmsClientVMId, String remotingClientSessionID);

   boolean containsSession(String remotingClientSessionID);

   /**
    * Returns a list of active connection endpoints currently maintained by an instance of this
    * manager. The implementation should make a copy of the list to avoid
    * ConcurrentModificationException. The list could be empty, but never null.
    *
    * @return List<ConnectionEndpoint>
    */
   List getActiveConnections();
   
   void handleClientFailure(String remotingSessionID);

}
