/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.jms.server.endpoint;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jboss.jms.message.JBossMessage;
import org.jboss.jms.message.MessageProxy;
import org.jboss.messaging.core.message.MessageFactory;

/**
 * 
 * A ClientDelivery
 * Encapsulates a delivery of some messages to a client consumer
 * 
 * @author <a href="mailto:tim.fox@jboss.com">Tim Fox</a>
 * @version <tt>$Revision: 2594 $</tt>
 *
 * $Id: ClientDelivery.java 2594 2007-04-20 15:44:23Z timfox $
 *
 */
public class ClientDelivery implements Externalizable
{
   // Constants -----------------------------------------------------
   
   private static final long serialVersionUID = 8375144805659344430L;

   // Static --------------------------------------------------------
   
   // Attributes ----------------------------------------------------
   
   private List msgs;
         
   private int consumerID;
   
   private int serverID;
    
   // Constructors --------------------------------------------------
   
   public ClientDelivery()
   {      
   }

   public ClientDelivery(List msgs, int consumerID, int serverID)
   {
      this.msgs = msgs;
      
      this.consumerID = consumerID; 
      
      this.serverID = serverID;
   }
  
   // Externalizable implementation
   // ---------------------------------------------------------------
   
   public void writeExternal(ObjectOutput out) throws IOException
   {
      out.writeInt(consumerID);
      
      out.writeInt(serverID);
      
      out.writeInt(msgs.size());
      
      Iterator iter = msgs.iterator();
      
      while (iter.hasNext())
      {
         MessageProxy mp = (MessageProxy)iter.next();
         
         out.writeByte(mp.getMessage().getType());

         out.writeInt(mp.getDeliveryCount());

         mp.getMessage().writeExternal(out);
      }      
   }

   public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
   {
      consumerID = in.readInt();
      
      serverID = in.readInt();
      
      int numMessages = in.readInt();
      
      msgs = new ArrayList(numMessages);
      
      for (int i = 0; i < numMessages; i++)
      {
         byte type = in.readByte();
         
         int deliveryCount = in.readInt();
         
         JBossMessage m = (JBossMessage)MessageFactory.createMessage(type);

         m.readExternal(in);

         MessageProxy md = JBossMessage.createThinDelegate(m, deliveryCount);
         
         msgs.add(md);
      }      
   }

   // Public --------------------------------------------------------
   
   public List getMessages()
   {
      return msgs;
   }
   
   public int getConsumerID()
   {
      return consumerID;
   }
   
   public int getServerID()
   {
      return serverID;
   }

   // Package protected ---------------------------------------------
   
   // Protected -----------------------------------------------------
   
   // Private -------------------------------------------------------
   
   // Inner classes -------------------------------------------------   
}
