/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.messaging.core;


import org.jboss.messaging.core.Receiver;
import org.jboss.messaging.core.Routable;
import org.jboss.messaging.core.Delivery;
import org.jboss.messaging.core.DeliveryObserver;
import org.jboss.messaging.core.SimpleDelivery;
import org.jboss.messaging.core.MessageReference;
import org.jboss.messaging.core.tx.Transaction;
import org.jboss.logging.Logger;

import java.util.List;
import java.util.ArrayList;


/**
 * A simple Receiver that "breaks" after the
 *
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 689 $</tt>
 *
 * $Id: BrokenReceiver.java 689 2006-03-01 23:05:11Z ovidiu $
 */
public class BrokenReceiver implements Receiver
{
   // Constants -----------------------------------------------------

   private static final Logger log = Logger.getLogger(BrokenReceiver.class);

   // Static --------------------------------------------------------

   // Attributes ----------------------------------------------------

   private int counter;
   private int failurePoint;
   private List messages;


   // Constructors --------------------------------------------------

   public BrokenReceiver(int failurePoint)
   {
      messages = new ArrayList();
      this.failurePoint = failurePoint;
   }

   // Receiver implementation ---------------------------------------

   public Delivery handle(DeliveryObserver observer, Routable r, Transaction tx)
   {
      counter++;

      if (counter == failurePoint)
      {
         throw new RuntimeException("I AM BROKEN!");
      }

      messages.add(r);
      return new SimpleDelivery(observer, (MessageReference)r, true);
   }

   // Public --------------------------------------------------------

   public List getMessages()
   {
      return messages;
   }

   public String toString()
   {
      return "BrokenReceiver";
   }

   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

}
