/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.messaging.core.distributed.queue;

import org.jboss.messaging.core.distributed.queue.DistributedQueue;
import org.jboss.messaging.core.local.Queue;
import org.jboss.messaging.core.plugin.JDBCPersistenceManager;
import org.jboss.messaging.core.plugin.SimpleMessageStore;
import org.jboss.messaging.core.plugin.contract.PersistenceManager;
import org.jboss.test.messaging.core.distributed.queue.base.DistributedQueueTestBase;

import EDU.oswego.cs.dl.util.concurrent.QueuedExecutor;

/**
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 1019 $</tt>
 *
 * $Id: RecoverableDistributedQueueTest.java 1019 2006-07-17 17:15:04Z timfox $
 */
public class RecoverableDistributedQueueTest extends DistributedQueueTestBase
{
   // Constants -----------------------------------------------------

   // Static --------------------------------------------------------
   
   // Attributes ----------------------------------------------------

   private PersistenceManager tl;
   private PersistenceManager tl2;
   private PersistenceManager tl3;

   // Constructors --------------------------------------------------

    public RecoverableDistributedQueueTest(String name)
   {
      super(name);
   }

   // Public --------------------------------------------------------

   public void setUp() throws Exception
   {
      super.setUp();

      //    FIXME - Why are we starting more than one persistence manager using the same datasource??
      //They will pointing at the same set of db tables thus giving indeterministic results
      
      
      tl = new JDBCPersistenceManager(sc.getDataSource(), sc.getTransactionManager());
      tl.start();
      tl2 = new JDBCPersistenceManager(sc.getDataSource(), sc.getTransactionManager());
      tl2.start();
      tl3 = new JDBCPersistenceManager(sc.getDataSource(), sc.getTransactionManager());
      tl3.start();

      ms = new SimpleMessageStore("s50");

      ms2 = new SimpleMessageStore("s51");

      ms3 = new SimpleMessageStore("s52");

      channel = new DistributedQueue("test", ms, tl, dispatcher);
      channel2 = new DistributedQueue("test", ms2, tl2, dispatcher2);
      channel3 = new DistributedQueue("test", ms3, tl3, dispatcher3);

      // initialize the transaction repository with the new transaction log
      tr.start(tl);

      log.debug("setup done");
   }

   public void tearDown() throws Exception
   {
      channel.close();
      channel = null;

      channel2.close();
      channel2 = null;

      channel3.close();
      channel3 = null;

      tl.stop();
      ms = null;

      tl2.stop();
      ms2 = null;

      tl3.stop();
      ms3 = null;

      super.tearDown();
   }

   public void crashChannel() throws Exception
   {
      channel.close();
      channel = null;

   }

   public void recoverChannel() throws Exception
   {
      channel = new Queue(-1, ms, tl, null, true, 0, 0, 0, new QueuedExecutor());
   }


   // Package protected ---------------------------------------------
   
   // Protected -----------------------------------------------------
   
   // Private -------------------------------------------------------
   
   // Inner classes -------------------------------------------------   
}
